@@ -74,9 +74,9 @@ import hashlib
 import json
 import os
 import posixpath
-import random
 import shutil
 import sys
+from dataclasses import dataclass
 from subprocess import PIPE
 from textwrap import dedent
 from typing import List
@@ -85,22 +85,11 @@ __scriptdir__ = os.path.dirname(os.path.abspath(__file__))
 __rootdir__ = os.path.dirname(__scriptdir__)
 sys.path.insert(0, __rootdir__)
 
-from tools import shared, utils, js_manipulation, diagnostics
+from tools import diagnostics, js_manipulation, shared, utils
 from tools.response_file import substitute_response_files
 
-
 DEBUG = os.environ.get('EMCC_DEBUG')
 
-IMAGE_SUFFIXES = ('.jpg', '.png', '.bmp')
-AUDIO_SUFFIXES = ('.ogg', '.wav', '.mp3')
-AUDIO_MIMETYPES = {'ogg': 'audio/ogg', 'wav': 'audio/wav', 'mp3': 'audio/mpeg'}
-
-DDS_HEADER_SIZE = 128
-
-# Set to 1 to randomize file order and add some padding,
-# to work around silly av false positives
-AV_WORKAROUND = 0
-
 excluded_patterns: List[str] = []
 new_data_files = []
 walked = []
@@ -134,12 +123,12 @@ class Options:
     self.export_es6 = False
 
 
+@dataclass
 class DataFile:
-  def __init__(self, srcpath, dstpath, mode, explicit_dst_path):
-    self.srcpath = srcpath
-    self.dstpath = dstpath
-    self.mode = mode
-    self.explicit_dst_path = explicit_dst_path
+  srcpath: str
+  dstpath: str
+  mode: str
+  explicit_dst_path: bool
 
 
 options = Options()
@@ -265,7 +254,7 @@ def generate_object_file(data_files):
   embed_files = [f for f in data_files if f.mode == 'embed']
   assert embed_files
 
-  asm_file = shared.replace_suffix(options.obj_output, '.s')
+  asm_file = utils.replace_suffix(options.obj_output, '.s')
 
   used = set()
   for f in embed_files:
@@ -373,7 +362,7 @@ def main():  # noqa: C901, PLR0912, PLR0915
   try:
     args = substitute_response_files(sys.argv[1:])
   except OSError as e:
-    shared.exit_with_error(e)
+    utils.exit_with_error(e)
 
   if '--help' in args:
     print(__doc__.strip())
@@ -381,7 +370,6 @@ def main():  # noqa: C901, PLR0912, PLR0915
 
   data_target = args[0]
   data_files = []
-  plugins = []
   leading = ''
 
   for arg in args[1:]:
@@ -438,10 +426,6 @@ def main():  # noqa: C901, PLR0912, PLR0915
       leading = ''
     elif arg == '--quiet':
       options.quiet = True
-    elif arg.startswith('--plugin'):
-      plugin = utils.read_file(arg.split('=', 1)[1])
-      eval(plugin) # should append itself to plugins
-      leading = ''
     elif leading in {'preload', 'embed'}:
       mode = leading
       # position of @ if we're doing 'src@dst'. '__' is used to keep the index
@@ -462,12 +446,10 @@ def main():  # noqa: C901, PLR0912, PLR0915
                                    explicit_dst_path=uses_at_notation))
       else:
         diagnostics.error(f'${arg} does not exist')
-        return 1
     elif leading == 'exclude':
       excluded_patterns.append(arg)
     else:
       diagnostics.error('Unknown parameter:', arg)
-      return 1
 
   options.has_preloaded = any(f.mode == 'preload' for f in data_files)
   options.has_embedded = any(f.mode == 'embed' for f in data_files)
@@ -477,11 +459,8 @@ def main():  # noqa: C901, PLR0912, PLR0915
         'for deprecation.  If you need this feature please comment at '
         'https://github.com/emscripten-core/emscripten/issues/24803')
 
-  if options.separate_metadata:
-    if not options.has_preloaded or not options.jsoutput:
-      err('cannot separate-metadata without both --preloaded files '
-          'and a specified --js-output')
-      return 1
+  if options.separate_metadata and (not options.has_preloaded or not options.jsoutput):
+    diagnostics.error('cannot separate-metadata without both --preloaded files and a specified --js-output')
 
   if not options.from_emcc and not options.quiet:
     diagnostics.warn('Remember to build the main file with `-sFORCE_FILESYSTEM` '
@@ -489,12 +468,9 @@ def main():  # noqa: C901, PLR0912, PLR0915
 
   if options.jsoutput and os.path.abspath(options.jsoutput) == os.path.abspath(data_target):
     diagnostics.error('TARGET should not be the same value of --js-output')
-    return 1
 
   if options.from_emcc and options.export_es6:
-    diagnostics.error('Can\'t use --export-es6 option together with --from-emcc since the code should be embedded '
-        'within emcc\'s code')
-    return 1
+    diagnostics.error("Can't use --export-es6 option together with --from-emcc since the code should be embedded within emcc's code")
 
   walked.append(__file__)
   for file_ in data_files:
@@ -508,7 +484,6 @@ def main():  # noqa: C901, PLR0912, PLR0915
                 if not os.path.isdir(file_.srcpath)]
   if len(data_files) == 0:
     diagnostics.error('Nothing to do!')
-    sys.exit(1)
 
   # Absolutize paths, and check that they make sense
   # os.getcwd() always returns the hard path with any symbolic links resolved,
@@ -568,27 +543,35 @@ def main():  # noqa: C901, PLR0912, PLR0915
   data_files = sorted(data_files, key=lambda file_: file_.dstpath)
   data_files = [file_ for file_ in data_files if not was_seen(file_.dstpath)]
 
-  if AV_WORKAROUND:
-    random.shuffle(data_files)
-
-  # Apply plugins
-  for file_ in data_files:
-    for plugin in plugins:
-      plugin(file_)
-
   metadata = {'files': []}
 
+  if options.depfile:
+    targets = []
+    if options.obj_output:
+      targets.append(options.obj_output)
+    if options.jsoutput:
+      targets.append(data_target)
+      targets.append(options.jsoutput)
+    with open(options.depfile, 'w') as f:
+      for target in targets:
+        if target:
+          f.write(escape_for_makefile(target))
+          f.write(' \\\n')
+      f.write(': \\\n')
+      for dependency in walked:
+        f.write(escape_for_makefile(dependency))
+        f.write(' \\\n')
+
   if options.obj_output:
     if not options.has_embedded:
       diagnostics.error('--obj-output is only applicable when embedding files')
-      return 1
     generate_object_file(data_files)
     if not options.has_preloaded:
       return 0
 
   ret = generate_js(data_target, data_files, metadata)
 
-  if options.force or len(data_files):
+  if options.force or data_files:
     if options.jsoutput is None:
       print(ret)
     else:
@@ -604,17 +587,6 @@ def main():  # noqa: C901, PLR0912, PLR0915
       if options.separate_metadata:
         utils.write_file(options.jsoutput + '.metadata', json.dumps(metadata, separators=(',', ':')))
 
-  if options.depfile:
-    with open(options.depfile, 'w') as f:
-      for target in (data_target, options.jsoutput):
-        if target:
-          f.write(escape_for_makefile(target))
-          f.write(' \\\n')
-      f.write(': \\\n')
-      for dependency in walked:
-        f.write(escape_for_makefile(dependency))
-        f.write(' \\\n')
-
   return 0
 
 
@@ -639,7 +611,7 @@ def generate_js(data_target, data_files, metadata):
   var Module = typeof %(EXPORT_NAME)s != 'undefined' ? %(EXPORT_NAME)s : {};\n''' % {"EXPORT_NAME": options.export_name}
 
   ret += '''
-  Module['expectedDataFileDownloads'] ??= 0;
+  if (!Module['expectedDataFileDownloads']) Module['expectedDataFileDownloads'] = 0;
   Module['expectedDataFileDownloads']++;'''
 
   if not options.export_es6:
@@ -653,7 +625,7 @@ def generate_js(data_target, data_files, metadata):
     if (isPthread || isWasmWorker) return;\n'''
 
   if options.support_node:
-    ret += "    var isNode = typeof process === 'object' && typeof process.versions === 'object' && typeof process.versions.node === 'string';\n"
+    ret += "    var isNode = globalThis.process && globalThis.process.versions && globalThis.process.versions.node && globalThis.process.type != 'renderer';\n"
 
   if options.support_node and options.export_es6:
         ret += '''if (isNode) {
@@ -694,8 +666,6 @@ def generate_js(data_target, data_files, metadata):
         file_.data_start = start
         curr = utils.read_binary(file_.srcpath)
         file_.data_end = start + len(curr)
-        if AV_WORKAROUND:
-            curr += '\x00'
         start += len(curr)
         data.write(curr)
 
@@ -708,52 +678,25 @@ def generate_js(data_target, data_files, metadata):
     create_preloaded = '''
           try {
             // canOwn this data in the filesystem, it is a slice into the heap that will never change
-            await Module['FS_preloadFile'](this.name, null, byteArray, true, true, false, true);
-            Module['removeRunDependency'](`fp ${that.name}`);
+            await Module['FS_preloadFile'](name, null, data, true, true, false, true);
+            Module['removeRunDependency'](`fp ${name}`);
           } catch (e) {
-            err(`Preloading file ${that.name} failed`);
+            err(`Preloading file ${name} failed`, e);
           }\n'''
     create_data = '''// canOwn this data in the filesystem, it is a slice into the heap that will never change
-          Module['FS_createDataFile'](this.name, null, byteArray, true, true, true);
-          Module['removeRunDependency'](`fp ${that.name}`);'''
+          Module['FS_createDataFile'](name, null, data, true, true, true);
+          Module['removeRunDependency'](`fp ${name}`);'''
 
     finish_handler = create_preloaded if options.use_preload_plugins else create_data
-    if options.export_es6:
-      finish_handler += '\nloadDataResolve();'
 
     if not options.lz4:
       # Data requests - for getting a block of data out of the big archive - have
       # a similar API to XHRs
       code += '''
-      /** @constructor */
-      function DataRequest(start, end, audio) {
-        this.start = start;
-        this.end = end;
-        this.audio = audio;
-      }
-      DataRequest.prototype = {
-        requests: {},
-        open: function(mode, name) {
-          this.name = name;
-          this.requests[name] = this;
-          Module['addRunDependency'](`fp ${this.name}`);
-        },
-        send: function() {},
-        onload: function() {
-          var byteArray = this.byteArray.subarray(this.start, this.end);
-          this.finish(byteArray);
-        },
-        finish: async function(byteArray) {
-          var that = this;
-          %s
-          this.requests[this.name] = null;
-        }
-      };
-
-      var files = metadata['files'];
-      for (var i = 0; i < files.length; ++i) {
-        new DataRequest(files[i]['start'], files[i]['end'], files[i]['audio'] || 0).open('GET', files[i]['filename']);
-      }\n''' % finish_handler
+      for (var file of metadata['files']) {
+        var name = file['filename']
+        Module['addRunDependency'](`fp ${name}`);
+      }\n'''
 
   if options.has_embedded and not options.obj_output:
     diagnostics.warn('--obj-output is recommended when using --embed.  This outputs an object file for linking directly into your application is more efficient than JS encoding')
@@ -779,15 +722,11 @@ def generate_js(data_target, data_files, metadata):
                  % (dirname, basename, counter))
     elif file_.mode == 'preload':
       # Preload
-      metadata_el = {
+      metadata['files'].append({
         'filename': file_.dstpath,
         'start': file_.data_start,
         'end': file_.data_end,
-      }
-      if filename[-4:] in AUDIO_SUFFIXES:
-        metadata_el['audio'] = 1
-
-      metadata['files'].append(metadata_el)
+      })
     else:
       assert 0
 
@@ -795,14 +734,13 @@ def generate_js(data_target, data_files, metadata):
     if not options.lz4:
       # Get the big archive and split it up
       use_data = '''// Reuse the bytearray from the XHR as the source for file reads.
-          DataRequest.prototype.byteArray = byteArray;
-          var files = metadata['files'];
-          for (var i = 0; i < files.length; ++i) {
-            DataRequest.prototype.requests[files[i].filename].onload();
-          }'''
-      use_data += ("          Module['removeRunDependency']('datafile_%s');\n"
-                   % js_manipulation.escape_for_js_string(data_target))
-
+          for (var file of metadata['files']) {
+            var name = file['filename'];
+            var data = byteArray.subarray(file['start'], file['end']);
+            %s
+          }
+          Module['removeRunDependency']('datafile_%s');''' % (finish_handler,
+                                                              js_manipulation.escape_for_js_string(data_target))
     else:
       # LZ4FS usage
       temp = data_target + '.orig'
@@ -816,6 +754,9 @@ def generate_js(data_target, data_files, metadata):
             Module['LZ4'].loadPackage({ 'metadata': metadata, 'compressedData': compressedData }, %s);
             Module['removeRunDependency']('datafile_%s');''' % (meta, "true" if options.use_preload_plugins else "false", js_manipulation.escape_for_js_string(data_target))
 
+    if options.export_es6:
+      use_data += '\nloadDataResolve();'
+
     package_name = data_target
     remote_package_size = os.path.getsize(package_name)
     remote_package_name = os.path.basename(package_name)
@@ -829,7 +770,7 @@ def generate_js(data_target, data_files, metadata):
       }
       var PACKAGE_NAME = '%s';
       var REMOTE_PACKAGE_BASE = '%s';
-      var REMOTE_PACKAGE_NAME = Module['locateFile']?.(REMOTE_PACKAGE_BASE, '') ?? REMOTE_PACKAGE_BASE;\n''' % (js_manipulation.escape_for_js_string(data_target), js_manipulation.escape_for_js_string(remote_package_name))
+      var REMOTE_PACKAGE_NAME = Module['locateFile'] ? Module['locateFile'](REMOTE_PACKAGE_BASE, '') : REMOTE_PACKAGE_BASE;\n''' % (js_manipulation.escape_for_js_string(data_target), js_manipulation.escape_for_js_string(remote_package_name))
     metadata['remote_package_size'] = remote_package_size
     ret += "      var REMOTE_PACKAGE_SIZE = metadata['remote_package_size'];\n"
 
@@ -996,15 +937,14 @@ def generate_js(data_target, data_files, metadata):
     if options.support_node:
       node_support_code = '''
         if (isNode) {
-          var fsPromises = require('fs/promises');
-          var contents = await fsPromises.readFile(packageName);
-          return contents.buffer;
+          var contents = require('fs').readFileSync(packageName);
+          return new Uint8Array(contents).buffer;
         }'''.strip()
 
     ret += '''
       async function fetchRemotePackage(packageName, packageSize) {
         %(node_support_code)s
-        Module['dataFileDownloads'] ??= {};
+        if (!Module['dataFileDownloads']) Module['dataFileDownloads'] = {};
         try {
           var response = await fetch(packageName);
         } catch (e) {
@@ -1016,10 +956,10 @@ def generate_js(data_target, data_files, metadata):
 
         const chunks = [];
         const headers = response.headers;
-        const total = Number(headers.get('Content-Length') ?? packageSize);
+        const total = Number(headers.get('Content-Length') || packageSize);
         let loaded = 0;
 
-        Module['setStatus']?.('Downloading data...');
+        Module['setStatus'] && Module['setStatus']('Downloading data...');
         const reader = response.body.getReader();
 
         while (1) {
@@ -1037,7 +977,7 @@ def generate_js(data_target, data_files, metadata):
             totalSize += download.total;
           }
 
-          Module['setStatus']?.(`Downloading data... (${totalLoaded}/${totalSize})`);
+          Module['setStatus'] && Module['setStatus'](`Downloading data... (${totalLoaded}/${totalSize})`);
         }
 
         const packageData = new Uint8Array(chunks.map((c) => c.length).reduce((a, b) => a + b, 0));
@@ -1050,9 +990,9 @@ def generate_js(data_target, data_files, metadata):
       }\n''' % {'node_support_code': node_support_code}
 
     code += '''
-      function processPackageData(arrayBuffer) {
+      async function processPackageData(arrayBuffer) {
         assert(arrayBuffer, 'Loading data file failed.');
-        assert(arrayBuffer.constructor.name === ArrayBuffer.name, 'bad input to processPackageData');
+        assert(arrayBuffer.constructor.name === ArrayBuffer.name, 'bad input to processPackageData ' + arrayBuffer.constructor.name);
         var byteArray = new Uint8Array(arrayBuffer);
         var curr;
         %s
@@ -1062,7 +1002,7 @@ def generate_js(data_target, data_files, metadata):
     # we need to find the datafile in the same dir as the html file
 
     code += '''
-      Module['preloadResults'] ??= {};\n'''
+      if (!Module['preloadResults']) Module['preloadResults'] = {};\n'''
 
     if options.use_preload_cache:
       code += '''
@@ -1092,38 +1032,28 @@ def generate_js(data_target, data_files, metadata):
           await preloadFallback(e)%s;
         }
 
-        Module['setStatus']?.('Downloading...');\n''' % catch_handler
+        Module['setStatus'] && Module['setStatus']('Downloading...');\n''' % catch_handler
     else:
       # Not using preload cache, so we might as well start the xhr ASAP,
       # potentially before JS parsing of the main codebase if it's after us.
       # Only tricky bit is the fetch is async, but also when runWithFS is called
       # is async, so we handle both orderings.
       ret += '''
-      var fetchedCallback;
+      var fetchPromise;
       var fetched = Module['getPreloadedPackage']?.(REMOTE_PACKAGE_NAME, REMOTE_PACKAGE_SIZE);
 
       if (!fetched) {
         // Note that we don't use await here because we want to execute the
         // the rest of this function immediately.
-        fetchRemotePackage(REMOTE_PACKAGE_NAME, REMOTE_PACKAGE_SIZE)
-          .then((data) => {
-            if (fetchedCallback) {
-              fetchedCallback(data);
-              fetchedCallback = null;
-            } else {
-              fetched = data;
-            }
-          })%s;
+        fetchPromise = fetchRemotePackage(REMOTE_PACKAGE_NAME, REMOTE_PACKAGE_SIZE)%s;
       }\n''' % catch_handler
 
       code += '''
       Module['preloadResults'][PACKAGE_NAME] = {fromCache: false};
-      if (fetched) {
-        processPackageData(fetched);
-        fetched = null;
-      } else {
-        fetchedCallback = processPackageData;
-      }\n'''
+      if (!fetched) {
+        fetched = await fetchPromise;
+      }
+      processPackageData(fetched);\n'''
 
   ret += '''
     async function runWithFS(Module) {\n'''
@@ -1133,7 +1063,8 @@ def generate_js(data_target, data_files, metadata):
     if (Module['calledRun']) {
       runWithFS(Module)%s;
     } else {
-      (Module['preRun'] ??= []).push(runWithFS); // FS is not initialized yet, wait for it
+      if (!Module['preRun']) Module['preRun'] = [];
+      Module['preRun'].push(runWithFS); // FS is not initialized yet, wait for it
     }\n''' % catch_handler
 
   if options.separate_metadata:
@@ -1141,9 +1072,12 @@ def generate_js(data_target, data_files, metadata):
     if options.support_node:
       node_support_code = '''
         if (isNode) {
-          var fsPromises = require('fs/promises');
-          var contents = await fsPromises.readFile(metadataUrl, 'utf8');
-          return loadPackage(JSON.parse(contents));
+          var contents = require('fs').readFileSync(metadataUrl, 'utf8');
+          // The await here is needed, even though JSON.parse is a sync API.  It works
+          // around a issue with `removeRunDependency` otherwise being called to early
+          // on the metadata object.
+          var json = await JSON.parse(contents);
+          return loadPackage(json);
         }'''.strip()
 
     ret += '''
@@ -1152,20 +1086,21 @@ def generate_js(data_target, data_files, metadata):
 
   async function runMetaWithFS() {
     Module['addRunDependency']('%(metadata_file)s');
-    var metadataUrl = Module['locateFile']?.('%(metadata_file)s', '') ?? '%(metadata_file)s';
+    var metadataUrl = Module['locateFile'] ? Module['locateFile']('%(metadata_file)s', '') : '%(metadata_file)s';
     %(node_support_code)s
     var response = await fetch(metadataUrl);
     if (!response.ok) {
       throw new Error(`${response.status}: ${response.url}`);
     }
     var json = await response.json();
-    return loadPackage(json);
+    await loadPackage(json);
   }
 
   if (Module['calledRun']) {
     runMetaWithFS();
   } else {
-    (Module['preRun'] ??= []).push(runMetaWithFS);
+    if (!Module['preRun']) Module['preRun'] = [];
+    Module['preRun'].push(runMetaWithFS);
   }\n''' % {'node_support_code': node_support_code, 'metadata_file': os.path.basename(options.jsoutput + '.metadata')}
   else:
     ret += '''
