@@ -3,10 +3,16 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
+"""General purpose utility functions.  The code in this file should mostly be
+not emscripten-specific, but general purpose enough to be useful in any command
+line utility."""
+
+import functools
+import logging
 import os
 import shutil
+import stat
 import sys
-from functools import wraps
 from pathlib import Path
 
 from . import diagnostics
@@ -16,6 +22,8 @@ WINDOWS = sys.platform.startswith('win')
 MACOS = sys.platform == 'darwin'
 LINUX = sys.platform.startswith('linux')
 
+logger = logging.getLogger('utils')
+
 
 def exit_with_error(msg, *args):
   diagnostics.error(msg, *args)
@@ -25,6 +33,46 @@ def path_from_root(*pathelems):
   return str(Path(__rootpath__, *pathelems))
 
 
+def suffix(name):
+  """Return the file extension"""
+  return os.path.splitext(name)[1]
+
+
+def exe_suffix(cmd):
+  return cmd + '.exe' if WINDOWS else cmd
+
+
+def bat_suffix(cmd):
+  return cmd + '.bat' if WINDOWS else cmd
+
+
+def replace_suffix(filename, new_suffix):
+  assert new_suffix[0] == '.'
+  return os.path.splitext(filename)[0] + new_suffix
+
+
+def unsuffixed(name):
+  """Return the filename without the extension.
+
+  If there are multiple extensions this strips only the final one.
+  """
+  return os.path.splitext(name)[0]
+
+
+def unsuffixed_basename(name):
+  return os.path.basename(unsuffixed(name))
+
+
+def get_file_suffix(filename):
+  """Parses the essential suffix of a filename, discarding Unix-style version
+  numbers in the name. For example for 'libz.so.1.2.8' returns '.so'"""
+  while filename:
+    filename, suffix = os.path.splitext(filename)
+    if not suffix[1:].isdigit():
+      return suffix
+  return ''
+
+
 def normalize_path(path):
   """Normalize path separators to UNIX-style forward slashes.
 
@@ -40,6 +88,29 @@ def safe_ensure_dirs(dirname):
   os.makedirs(dirname, exist_ok=True)
 
 
+def make_writable(filename):
+  assert os.path.exists(filename)
+  old_mode = stat.S_IMODE(os.stat(filename).st_mode)
+  os.chmod(filename, old_mode | stat.S_IWUSR)
+
+
+def safe_copy(src, dst):
+  logger.debug('copy: %s -> %s', src, dst)
+  src = os.path.abspath(src)
+  dst = os.path.abspath(dst)
+  if os.path.isdir(dst):
+    dst = os.path.join(dst, os.path.basename(src))
+  if src == dst:
+    return
+  if dst == os.devnull:
+    return
+  # Copies data and permission bits, but not other metadata such as timestamp
+  shutil.copy(src, dst)
+  # We always want the target file to be writable even when copying from
+  # read-only source. (e.g. a read-only install of emscripten).
+  make_writable(dst)
+
+
 # TODO(sbc): Replace with str.removeprefix once we update to python3.9
 def removeprefix(string, prefix):
   if string.startswith(prefix):
@@ -49,6 +120,7 @@ def removeprefix(string, prefix):
 
 def convert_line_endings_in_file(filename, to_eol):
   if to_eol == os.linesep:
+    assert os.path.exists(filename)
     return # No conversion needed
 
   text = read_file(filename)
@@ -111,19 +183,22 @@ def delete_contents(dirname, exclude=None):
       delete_file(entry)
 
 
-# TODO(sbc): Replace with functools.cache, once we update to python 3.7
-def memoize(func):
-  results = {}
+def get_num_cores():
+  # Prefer `os.process_cpu_count` when available (3.13 and above) since
+  # it takes into account thread affinity.
+  # Fall back to `os.sched_getaffinity` where available and finally
+  # `os.cpu_count`, which should work everywhere.
+  if hasattr(os, 'process_cpu_count'):
+    cpu_count = os.process_cpu_count()
+  elif hasattr(os, 'sched_getaffinity'):
+    cpu_count = len(os.sched_getaffinity(0))
+  else:
+    cpu_count = os.cpu_count()
+  return int(os.environ.get('EMCC_CORES', cpu_count))
 
-  @wraps(func)
-  def helper(*args, **kwargs):
-    assert not kwargs
-    key = (func.__name__, args)
-    if key not in results:
-      results[key] = func(*args)
-    return results[key]
 
-  return helper
+# TODO(sbc): Replace with functools.cache, once we update to python 3.9
+memoize = functools.lru_cache(maxsize=None)
 
 
 # TODO: Move this back to shared.py once importing that file becoming side effect free (i.e. it no longer requires a config).
